#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include <UniversalTelegramBot.h>
#include <ArduinoJson.h>

// Замените на свои сетевые данные
char ssid[] = "XXXXXXXXX";         // Имя сети
char password[] = "XXXXXXXXXXX"; // Пароль
// Инициализация Telegram бота
#define TELEGRAM_BOT_TOKEN "XXXXXXXXXX:XXXXXXXXXXXXXXXX"  // Ваш Токен

// клиент Wi-Fi, поддерживает HTTPS
WiFiClientSecure client;
UniversalTelegramBot bot(TELEGRAM_BOT_TOKEN, client);

#define LED_PIN D5                    //Светодиод на плате подключено к D5 

int delayBetweenChecks = 1000;      // среднее время между сообщениями сканирования
unsigned long lastTimeChecked;          // когда было сделано последнее сканирование сообщений

unsigned long lightTimerExpires;
boolean lightTimerActive = false;

void setup() {
  Serial.begin(115200);

  // Установите Wi-Fi в режим станции
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(100);

  pinMode(D5, OUTPUT);
  digitalWrite(D5, HIGH);

  // попытка подключения к сети Wi-Fi
  Serial.print("Подключение WiFi: ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
  }

  Serial.println("");
  Serial.println("WiFi соединение");
  Serial.print("IP адрес: ");
  Serial.println(WiFi.localIP());

  // Работает на версии 2.5 Beta
  client.setInsecure();

  
// longPoll сохраняет запрос в Telegram открытым в течение заданного количества секунд,
// Применимо когда запрос исходит от Telegram, так как запрос блокирует цикл
  bot.longPoll = 20; // было 60
}

void handleNewMessages(int numNewMessages) {

  for (int i = 0; i < numNewMessages; i++) {

    // If the type is a "callback_query", a inline keyboard button was pressed
    if (bot.messages[i].type ==  F("callback_query")) {
      String text = bot.messages[i].text;
      Serial.print("Кнопка: ");
      Serial.println(text);

      if (text == F("ON")) {
        digitalWrite(LED_PIN, LOW);
      } else if (text == F("OFF")) {
        digitalWrite(LED_PIN, HIGH);
      } else if (text.startsWith("TIME")) {
        text.replace("TIME", "");
        int timeRequested = text.toInt();
        
        digitalWrite(LED_PIN, HIGH); 
        lightTimerActive = true;
        lightTimerExpires = millis() + (timeRequested * 1000 * 60);
      }
    } else {
      String chat_id = String(bot.messages[i].chat_id);
      String text = bot.messages[i].text;

      String from_name = bot.messages[i].from_name;
      if (from_name == "")
        from_name = "Guest";
        
        if (text == "/keyboard")
        {
          String keyboardJson = F("[[{ \"text\" : \"перейти на мой канал\", \"url\" : \"https://youtube.com/ардуинодляначинающих\" }],");
                 keyboardJson += F("[{ \"text\" : \"ВКЛ\", \"callback_data\" : \"ON\" },{ \"text\" : \"ВЫКЛ\", \"callback_data\" : \"OFF\"}],");
                 keyboardJson += F("[{ \"text\" : \"1 Mин\", \"callback_data\" : \"TIME1\" }, { \"text\" : \"2 Mин\", \"callback_data\" : \"TIME2\" }, { \"text\" : \"10 Mин\", \"callback_data\" : \"TIME10\" }]]");
          bot.sendMessageWithInlineKeyboard(chat_id, "Выберите любую кнопку", "", keyboardJson);
        }
  
        if (text == "/start")
        {
          String welcome = "Добро пожаловать в мой чатбот Телеграм, " + from_name + ".\n";
          welcome += "Это пример создания кнопок.\n\n";
          welcome += "/keyboard : Вывести кнопки\n";
  
          bot.sendMessage(chat_id, welcome, "Markdown");
        }
    }
  }
}

void loop() {
  if (millis() > lastTimeChecked + delayBetweenChecks)  {

    //  возвращает 1 если есть новое сообщение от Telegram
    int numNewMessages = bot.getUpdates(bot.last_message_received + 1);

    if (numNewMessages) {
      Serial.println("got response");
      handleNewMessages(numNewMessages);
    }

    lastTimeChecked = millis();

    if (lightTimerActive && millis() > lightTimerExpires) {
      lightTimerActive = false;
      digitalWrite(LED_PIN, LOW);
    }
  }
}
